// Intervallschalter
// von Nils Hintze

#include <avr/io.h>
#include <avr/interrupt.h>

uint8_t prescaler = 0;
uint16_t seconds = 0;
uint8_t output = 1;

int main() {
	// PB0: Ausschaltdauer Wert 2
	// PB1: Ausschaltdauer Wert 4
	// PB2: Ausschaltdauer Wert 1
	// PB3: Ausschaltdauer Wert 8
	// PB4: Einschaltdauer Wert 2
	// PB5: Einschaltdauer Wert 4
	// PB6: Einschaltdauer Wert 1
	// PB7: Einschaltdauer Wert 8
	DDRB =	0b00000000;
	PORTB =	0b11111111;

	// PD6: Ausgang
	DDRD =	0b1000000;
	PORTD =	0b0111111;

	// Timer 0 mit Vorteiler 1024 laufen lassen (16 berlufe pro Sekunde bei 4194304 Hz Taktfrequenz)
	TCCR0B = (1<<CS02) | (1<<CS00);
	// Overflow-Interrupt aktivieren
	TIMSK |= (1<<TOIE0);

	// Interrupts global aktivieren
	sei();

	// Endlosschleife
	while (1);

	return 0;
}

ISR(TIMER0_OVF_vect) {
	uint8_t temp = 0, ontime = 0, offtime = 0;

	prescaler++;
	if (prescaler >= 16) {
		prescaler = 0;
		seconds++;
	}

	// Schalterstellungen einlesen
	temp = ~PINB;
	offtime = (temp&0x08) | ((temp<<1)&0x06) | ((temp>>2)&0x01);
	temp >>= 4;
	ontime = (temp&0x08) | ((temp<<1)&0x06) | ((temp>>2)&0x01);

	if (ontime == 0) { // Immer aus
		seconds = 0;
		output = 0;
	} else if (offtime == 0) { // Immer ein
		seconds = 0;
		output = 1;
	} else {
		if (output == 0) { // Ausgang ist ausgeschaltet
			if (seconds >= (offtime * 60)) { // Ausschaltdauer berschritten
				seconds = 0;
				output = 1;
			}
		} else { // Ausgang ist eingeschaltet
			if (seconds >= (ontime * 60)) { // Einschaltdauer berschritten
				seconds = 0;
				output = 0;
			}
		}
	}

	if (output != 0) { // Ausgang soll eingeschaltet werden
		PORTD |= (1<<PD6);
	} else { // Ausgang soll ausgeschaltet werden
		PORTD &= ~(1<<PD6);
	}	
}
